@@ -102,6 +102,7 @@ public:
 		set_command_for_extra_actions("animate-spin-right", operators::Animate(GUIConfiguration::ANIMATION_VARIANT_SPIN_RIGHT));
 		set_command_for_extra_actions("animate-spin-on-z-left", operators::Animate(GUIConfiguration::ANIMATION_VARIANT_SPIN_ON_Z_LEFT));
 		set_command_for_extra_actions("animate-spin-on-z-right", operators::Animate(GUIConfiguration::ANIMATION_VARIANT_SPIN_ON_Z_RIGHT));
+		set_command_for_extra_actions("animate-wiggle", operators::Animate(GUIConfiguration::ANIMATION_VARIANT_WIGGLE_LEFT));
 		set_command_for_extra_actions("export-view", operators::ExportView());
 		set_command_for_extra_actions("import-view", operators::ImportView());
 		set_command_for_extra_actions("hint-render-area-size", operators::HintRenderAreaSize());
@@ -296,6 +297,16 @@ public:
 				{
 					uv::ViewerApplication::instance().rotate(glm::vec3(0, 1, 0), 0.01);
 				}
+				else if(GUIConfiguration::instance().animation_variant==GUIConfiguration::ANIMATION_VARIANT_WIGGLE_LEFT)
+				{
+					uv::ViewerApplication::instance().rotate(glm::vec3(0, 1, 0), -0.02);
+					GUIConfiguration::instance().animation_variant=GUIConfiguration::ANIMATION_VARIANT_WIGGLE_RIGHT;
+				}
+				else if(GUIConfiguration::instance().animation_variant==GUIConfiguration::ANIMATION_VARIANT_WIGGLE_RIGHT)
+				{
+					uv::ViewerApplication::instance().rotate(glm::vec3(0, 1, 0), 0.02);
+					GUIConfiguration::instance().animation_variant=GUIConfiguration::ANIMATION_VARIANT_WIGGLE_LEFT;
+				}
 				else if(GUIConfiguration::instance().animation_variant==GUIConfiguration::ANIMATION_VARIANT_SPIN_ON_Z_LEFT)
 				{
 					uv::ViewerApplication::instance().rotate(glm::vec3(0, 0, 1), -0.01);
