@@ -21,7 +21,7 @@ namespace
 void print_help(const bool full, std::ostream& output) noexcept
 {
 	output << R"(
-Voronota-LT version 0.9.4
+Voronota-LT version 0.9.5
 
 'voronota-lt' executable constructs a radical Voronoi tessellation (also known as a Laguerre-Voronoi diagram or a power diagram)
 of atomic balls of van der Waals radii constrained inside a solvent-accessible surface defined by a rolling probe.
@@ -62,6 +62,8 @@ Options:
     --write-cells-to-file                            string     output file path to write of per-cell summaries
     --write-cells-residue-level-to-file              string     output file path to write residue-level grouped per-cell summaries
     --write-cells-chain-level-to-file                string     output file path to write chain-level grouped per-cell summaries
+    --write-tessellation-edges-to-file               string     output file path to write generating IDs and lengths of SAS-constrained tessellation edges
+    --write-tessellation-vertices-to-file            string     output file path to write generating IDs and positions of SAS-constrained tessellation vertices
     --graphics-output-file                           string     output file path to write contacts drawing .py script to run in PyMol)";
 
 	if(full)
@@ -186,6 +188,8 @@ public:
 	std::set<std::string> graphics_restrict_chains;
 	std::set< std::pair<std::string, std::string> > graphics_restrict_chain_pairs;
 	std::string mesh_output_obj_file;
+	std::string write_tessellation_edges_to_file;
+	std::string write_tessellation_vertices_to_file;
 	std::string write_log_to_file;
 	std::ostringstream error_log_for_options_parsing;
 
@@ -437,6 +441,14 @@ public:
 				{
 					mesh_extract_connected_component=static_cast<long>(opt.args_ints.front());
 				}
+				else if(opt.name=="write-tessellation-edges-to-file" && opt.args_strings.size()==1)
+				{
+					write_tessellation_edges_to_file=opt.args_strings.front();
+				}
+				else if(opt.name=="write-tessellation-vertices-to-file" && opt.args_strings.size()==1)
+				{
+					write_tessellation_vertices_to_file=opt.args_strings.front();
+				}
 				else if(opt.name=="write-log-to-file" && opt.args_strings.size()==1)
 				{
 					write_log_to_file=opt.args_strings.front();
@@ -511,6 +523,11 @@ public:
 			error_log_for_options_parsing << "Error: in this version mesh output and analysis is disabled for the simplified additively weighted Voronoi diagram regime.\n";
 		}
 
+		if(running_mode==RunningMode::simplified_aw && !(write_tessellation_edges_to_file.empty() && write_tessellation_vertices_to_file.empty()))
+		{
+			error_log_for_options_parsing << "Error: in this version tessellation edges and vertices output is disabled for the simplified additively weighted Voronoi diagram regime.\n";
+		}
+
 		if(!periodic_box_directions.empty() && !periodic_box_corners.empty())
 		{
 			error_log_for_options_parsing << "Error: cannot use both the periodic box directions and the periodic box corners.\n";
@@ -735,7 +752,15 @@ void run_mode_radical(
 		const std::vector<int>& grouping_for_filtering=(app_params.compute_only_inter_chain_contacts ? spheres_input_result.grouping_by_chain : (app_params.compute_only_inter_residue_contacts ? spheres_input_result.grouping_by_residue : null_grouping));
 		const bool summarize_cells=grouping_for_filtering.empty();
 
-		voronotalt::RadicalTessellation::construct_full_tessellation(spheres_container, grouping_for_filtering, (app_graphics_recorder.graphics_writer.enabled() || app_mesh_recorder.mesh_writer.enabled()), summarize_cells, result, result_graphics, app_log_recorders.time_recoder_for_tessellation);
+		voronotalt::RadicalTessellation::construct_full_tessellation(
+				spheres_container,
+				grouping_for_filtering,
+				!(app_params.write_tessellation_edges_to_file.empty() && app_params.write_tessellation_vertices_to_file.empty()),
+				(app_graphics_recorder.graphics_writer.enabled() || app_mesh_recorder.mesh_writer.enabled()),
+				summarize_cells,
+				result,
+				result_graphics,
+				app_log_recorders.time_recoder_for_tessellation);
 	}
 
 	voronotalt::RadicalTessellation::GroupedResult result_grouped_by_residue;
@@ -869,6 +894,34 @@ void run_mode_radical(
 
 	app_log_recorders.time_recoder_for_output.record_elapsed_miliseconds_and_reset("print result sas and volumes");
 
+	if(!app_params.write_tessellation_edges_to_file.empty())
+	{
+		std::ofstream foutput(app_params.write_tessellation_edges_to_file.c_str(), std::ios::out);
+		if(foutput.good())
+		{
+			voronotalt::PrintingCustomTypes::print_sequential_container_simply(result.tessellation_net.tes_edges, foutput);
+		}
+		else
+		{
+			std::cerr << "Error (non-terminating): failed to write tessellation edges to file '" << app_params.write_tessellation_edges_to_file << "'\n";
+		}
+		app_log_recorders.time_recoder_for_output.record_elapsed_miliseconds_and_reset("write tessellation edges");
+	}
+
+	if(!app_params.write_tessellation_vertices_to_file.empty())
+	{
+		std::ofstream foutput(app_params.write_tessellation_vertices_to_file.c_str(), std::ios::out);
+		if(foutput.good())
+		{
+			voronotalt::PrintingCustomTypes::print_sequential_container_simply(result.tessellation_net.tes_vertices, foutput);
+		}
+		else
+		{
+			std::cerr << "Error (non-terminating): failed to write tessellation vertices to file '" << app_params.write_tessellation_vertices_to_file << "'\n";
+		}
+		app_log_recorders.time_recoder_for_output.record_elapsed_miliseconds_and_reset("write tessellation vertices");
+	}
+
 	if(app_graphics_recorder.graphics_writer.enabled())
 	{
 		if(ApplicationGraphicsRecorder::allow_representation(app_params.graphics_restrict_representations, "balls"))
@@ -1504,7 +1557,7 @@ void run_mode_test_raw_collisions(
 		{
 			voronotalt::RadicalTessellation::ResultGraphics result_graphics;
 
-			voronotalt::RadicalTessellation::construct_full_tessellation(spheres_container, grouping_for_filtering, false, false, tessellation_result, result_graphics, app_log_recorders.time_recoder_for_tessellation);
+			voronotalt::RadicalTessellation::construct_full_tessellation(spheres_container, grouping_for_filtering, false, false, false, tessellation_result, result_graphics, app_log_recorders.time_recoder_for_tessellation);
 		}
 	}
 
