@@ -70,15 +70,26 @@ public:
 
 	struct ContactDescriptorSummaryAdjunct
 	{
+		struct LevelArea
+		{
+			UnsignedInt zone;
+			Float restriction;
+			Float area;
+
+			LevelArea() : zone(0), restriction(FLOATCONST(0.0)), area(FLOATCONST(0.0))
+			{
+			}
+		};
+
 		ContactDescriptorSummaryAdjunct() noexcept
 		{
 		}
 
-		explicit ContactDescriptorSummaryAdjunct(const UnsignedInt levels) noexcept : level_areas(levels, FLOATCONST(0.0))
+		explicit ContactDescriptorSummaryAdjunct(const UnsignedInt levels) noexcept : level_areas(levels)
 		{
 		}
 
-		std::vector<Float> level_areas;
+		std::vector<LevelArea> level_areas;
 	};
 
 	struct CellContactDescriptorsSummary
@@ -231,6 +242,18 @@ public:
 		}
 	};
 
+	struct TessellationNet
+	{
+		std::vector<RadicalTessellationContactConstruction::TessellationVertex> tes_vertices;
+		std::vector<RadicalTessellationContactConstruction::TessellationEdge> tes_edges;
+
+		void clear() noexcept
+		{
+			tes_vertices.clear();
+			tes_edges.clear();
+		}
+	};
+
 	struct Result
 	{
 		UnsignedInt total_spheres;
@@ -243,6 +266,7 @@ public:
 		TotalCellContactDescriptorsSummary total_cells_summary;
 		std::vector<ContactDescriptorSummary> contacts_summaries_with_redundancy_in_periodic_box;
 		std::vector<UnsignedInt> contacts_canonical_ids_with_redundancy_in_periodic_box;
+		TessellationNet tessellation_net;
 
 		Result() noexcept : total_spheres(0), total_collisions(0), total_relevant_collisions(0)
 		{
@@ -260,6 +284,7 @@ public:
 			total_cells_summary=TotalCellContactDescriptorsSummary();
 			contacts_summaries_with_redundancy_in_periodic_box.clear();
 			contacts_canonical_ids_with_redundancy_in_periodic_box.clear();
+			tessellation_net.clear();
 		}
 	};
 
@@ -284,6 +309,95 @@ public:
 		}
 	};
 
+	struct ParametersForPreliminaryCuts
+	{
+		struct Plane
+		{
+			SimplePoint normal;
+			SimplePoint center;
+		};
+
+		std::vector<int> permissions;
+		std::vector< std::vector<Plane> > cutting_planes;
+		bool apply_with_single_mask;
+		bool apply_with_all_masks;
+		UnsignedInt single_mask;
+
+		ParametersForPreliminaryCuts() noexcept : apply_with_single_mask(false), apply_with_all_masks(false), single_mask(0)
+		{
+		}
+
+		bool check_if_enabled_and_valid(const UnsignedInt number_of_input_spheres) const noexcept
+		{
+			bool enabled_and_valid=((apply_with_single_mask!=apply_with_all_masks) && !cutting_planes.empty() && permissions.size()==number_of_input_spheres);
+			for(UnsignedInt g=0;enabled_and_valid && g<cutting_planes.size();g++)
+			{
+				enabled_and_valid=(enabled_and_valid && cutting_planes[g].size()==number_of_input_spheres);
+			}
+			return enabled_and_valid;
+		}
+
+		UnsignedInt calculate_number_of_possible_masks() const noexcept
+		{
+			return (static_cast<UnsignedInt>(1) << (2*cutting_planes.size()));
+		}
+
+		UnsignedInt calculate_mask_class(const UnsignedInt mask) const noexcept
+		{
+			const UnsignedInt onebit=1;
+			UnsignedInt mask_a=0;
+			UnsignedInt mask_b=0;
+			for(UnsignedInt g=0;g<cutting_planes.size();g++)
+			{
+				const UnsignedInt offset_shift=g*2;
+				const UnsignedInt bit_a=((mask & (onebit << (offset_shift+0))))==0 ? 0 : 1;
+				const UnsignedInt bit_b=((mask & (onebit << (offset_shift+1))))==0 ? 0 : 1;
+				mask_a+=(bit_a << (offset_shift+0))+(bit_b << (offset_shift+1));
+				mask_b+=(bit_a << (offset_shift+1))+(bit_b << (offset_shift+0));
+			}
+			return std::min(mask_a, mask_b);
+		}
+
+		void prepare_input_for_preliminary_cuts(const std::vector<SimpleSphere>& populated_spheres, const UnsignedInt id_a, const UnsignedInt id_b,
+				const UnsignedInt request_mask,	RadicalTessellationContactConstruction::PreliminaryCuttingPlanes& preliminary_cutting_planes) const noexcept
+		{
+			if(!cutting_planes.empty() && !permissions.empty())
+			{
+				preliminary_cutting_planes.enabled=true;
+				preliminary_cutting_planes.normals.resize(2*cutting_planes.size());
+				preliminary_cutting_planes.centers.resize(preliminary_cutting_planes.normals.size());
+				preliminary_cutting_planes.override_statuses.resize(preliminary_cutting_planes.normals.size(), 0);
+				const UnsignedInt input_n=static_cast<UnsignedInt>(permissions.size());
+				const UnsignedInt raw_ids[2]={id_a, id_b};
+				const UnsignedInt ref_ids[2]={(id_a%input_n), (id_b%input_n)};
+				const UnsignedInt onebit=1;
+				const UnsignedInt mask=(request_mask % (onebit << (2*cutting_planes.size())));
+				for(UnsignedInt g=0;g<cutting_planes.size();g++)
+				{
+					for(UnsignedInt s=0;s<2;s++)
+					{
+						const UnsignedInt nid=(g*2+s);
+						const int switch_status=((mask & (onebit << nid))==0 ? 1 : -1);
+						if(permissions[ref_ids[s]]>0)
+						{
+							preliminary_cutting_planes.override_statuses[nid]=0;
+							preliminary_cutting_planes.normals[nid]=point_and_number_product(cutting_planes[g][ref_ids[s]].normal, static_cast<Float>(switch_status));
+							preliminary_cutting_planes.centers[nid]=sum_of_points(populated_spheres[raw_ids[s]].p, cutting_planes[g][ref_ids[s]].center);
+						}
+						else
+						{
+							preliminary_cutting_planes.override_statuses[nid]=0-switch_status;
+						}
+					}
+				}
+			}
+			else
+			{
+				preliminary_cutting_planes.enabled=false;
+			}
+		}
+	};
+
 	static void construct_full_tessellation(
 			const std::vector<SimpleSphere>& input_spheres,
 			Result& result) noexcept
@@ -292,7 +406,7 @@ public:
 		SpheresContainer spheres_container;
 		spheres_container.init(input_spheres, time_recorder);
 		ResultGraphics result_graphics;
-		construct_full_tessellation(spheres_container, std::vector<int>(), std::vector<int>(), false, true, FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
+		construct_full_tessellation(spheres_container, std::vector<int>(), std::vector<int>(), false, false, true, FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
 	}
 
 	static void construct_full_tessellation(
@@ -304,7 +418,7 @@ public:
 		SpheresContainer spheres_container;
 		spheres_container.init(input_spheres, time_recorder);
 		ResultGraphics result_graphics;
-		construct_full_tessellation(spheres_container, std::vector<int>(), grouping_of_spheres, false, grouping_of_spheres.empty(), FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
+		construct_full_tessellation(spheres_container, std::vector<int>(), grouping_of_spheres, false, false, grouping_of_spheres.empty(), FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
 	}
 
 	static void construct_full_tessellation(
@@ -316,7 +430,20 @@ public:
 		SpheresContainer spheres_container;
 		spheres_container.init(input_spheres, periodic_box, time_recorder);
 		ResultGraphics result_graphics;
-		construct_full_tessellation(spheres_container, std::vector<int>(), std::vector<int>(), false, true, FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
+		construct_full_tessellation(spheres_container, std::vector<int>(), std::vector<int>(), false, false, true, FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
+	}
+
+	static void construct_full_tessellation(
+			const std::vector<SimpleSphere>& input_spheres,
+			const PeriodicBox& periodic_box,
+			const bool with_tessellation_net,
+			Result& result) noexcept
+	{
+		TimeRecorder time_recorder;
+		SpheresContainer spheres_container;
+		spheres_container.init(input_spheres, periodic_box, time_recorder);
+		ResultGraphics result_graphics;
+		construct_full_tessellation(spheres_container, std::vector<int>(), std::vector<int>(), with_tessellation_net, false, true, FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
 	}
 
 	static void construct_full_tessellation(
@@ -329,25 +456,27 @@ public:
 		SpheresContainer spheres_container;
 		spheres_container.init(input_spheres, periodic_box, time_recorder);
 		ResultGraphics result_graphics;
-		construct_full_tessellation(spheres_container, std::vector<int>(), grouping_of_spheres, false, grouping_of_spheres.empty(), FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
+		construct_full_tessellation(spheres_container, std::vector<int>(), grouping_of_spheres, false, false, grouping_of_spheres.empty(), FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
 	}
 
 	static void construct_full_tessellation(
 			const SpheresContainer& spheres_container,
 			const std::vector<int>& grouping_of_spheres,
+			const bool with_tessellation_net,
 			const bool with_graphics,
 			const bool summarize_cells,
 			Result& result,
 			ResultGraphics& result_graphics,
 			TimeRecorder& time_recorder) noexcept
 	{
-		construct_full_tessellation(spheres_container, std::vector<int>(), grouping_of_spheres, with_graphics, summarize_cells, FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
+		construct_full_tessellation(spheres_container, std::vector<int>(), grouping_of_spheres, with_tessellation_net, with_graphics, summarize_cells, FLOATCONST(0.0), std::vector<Float>(), result, result_graphics, time_recorder);
 	}
 
 	static void construct_full_tessellation(
 			const SpheresContainer& spheres_container,
 			const std::vector<int>& involvement_of_spheres,
 			const std::vector<int>& grouping_of_spheres,
+			const bool with_tessellation_net,
 			const bool with_graphics,
 			const bool summarize_cells,
 			const Float max_circle_radius_restriction,
@@ -355,6 +484,23 @@ public:
 			Result& result,
 			ResultGraphics& result_graphics,
 			TimeRecorder& time_recorder) noexcept
+	{
+		construct_full_tessellation(spheres_container, involvement_of_spheres, grouping_of_spheres, with_tessellation_net, with_graphics, summarize_cells, max_circle_radius_restriction, adjunct_max_circle_radius_restrictions, ParametersForPreliminaryCuts(), result, result_graphics, time_recorder);
+	}
+
+	static void construct_full_tessellation(
+			const SpheresContainer& spheres_container,
+			const std::vector<int>& involvement_of_spheres,
+			const std::vector<int>& grouping_of_spheres,
+			const bool with_tessellation_net,
+			const bool with_graphics,
+			const bool summarize_cells,
+			const Float max_circle_radius_restriction,
+			const std::vector<Float>& adjunct_max_circle_radius_restrictions,
+			const ParametersForPreliminaryCuts& parameters_for_preliminary_cuts,
+			Result& result,
+			ResultGraphics& result_graphics,
+			TimeRecorder& time_recorder) noexcept
 	{
 		time_recorder.reset();
 
@@ -369,8 +515,17 @@ public:
 		result.total_collisions=spheres_container.total_collisions();
 		result.total_relevant_collisions=preparation_result.relevant_collision_ids.size();
 
+		const bool apply_preliminary_cuts_with_single_mask=(parameters_for_preliminary_cuts.apply_with_single_mask && parameters_for_preliminary_cuts.check_if_enabled_and_valid(spheres_container.input_spheres().size()));
+		const bool apply_preliminary_cuts_with_all_masks=(parameters_for_preliminary_cuts.apply_with_all_masks && parameters_for_preliminary_cuts.check_if_enabled_and_valid(spheres_container.input_spheres().size()));
+
 		std::vector<ContactDescriptorSummary> possible_contacts_summaries(preparation_result.relevant_collision_ids.size());
 
+		std::vector<TessellationNet> possible_tessellation_subnets;
+		if(with_tessellation_net)
+		{
+			possible_tessellation_subnets.resize(possible_contacts_summaries.size());
+		}
+
 		std::vector<RadicalTessellationContactConstruction::ContactDescriptorGraphics> possible_contacts_graphics;
 		if(with_graphics)
 		{
@@ -386,6 +541,8 @@ public:
 			RadicalTessellationContactConstruction::ContactDescriptor cd;
 			cd.contour.reserve(12);
 
+			RadicalTessellationContactConstruction::PreliminaryCuttingPlanes preliminary_cutting_planes;
+
 #ifdef VORONOTALT_OPENMP
 #pragma omp for
 #endif
@@ -393,6 +550,10 @@ public:
 			{
 				const UnsignedInt id_a=preparation_result.relevant_collision_ids[i].first;
 				const UnsignedInt id_b=preparation_result.relevant_collision_ids[i].second;
+				if(apply_preliminary_cuts_with_single_mask)
+				{
+					parameters_for_preliminary_cuts.prepare_input_for_preliminary_cuts(spheres_container.populated_spheres(), id_a, id_b, parameters_for_preliminary_cuts.single_mask, preliminary_cutting_planes);
+				}
 				if(RadicalTessellationContactConstruction::construct_contact_descriptor(
 						spheres_container.populated_spheres(),
 						spheres_container.all_exclusion_statuses(),
@@ -400,9 +561,14 @@ public:
 						id_b,
 						spheres_container.all_colliding_ids()[id_a],
 						max_circle_radius_restriction,
+						preliminary_cutting_planes,
 						cd))
 				{
 					possible_contacts_summaries[i].set(cd);
+					if(with_tessellation_net)
+					{
+						RadicalTessellationContactConstruction::construct_contact_descriptor_tessellation_vertices_and_edges(cd, possible_tessellation_subnets[i].tes_edges, possible_tessellation_subnets[i].tes_vertices);
+					}
 					if(with_graphics)
 					{
 						RadicalTessellationContactConstruction::construct_contact_descriptor_graphics(cd, 0.2, possible_contacts_graphics[i]);
@@ -452,8 +618,10 @@ public:
 
 		if(!adjunct_max_circle_radius_restrictions.empty())
 		{
+			const UnsignedInt number_of_applications_of_preliminary_cuts=(!apply_preliminary_cuts_with_all_masks ? static_cast<UnsignedInt>(1) : parameters_for_preliminary_cuts.calculate_number_of_possible_masks());
+
 			result.adjuncts_for_contacts_summaries.clear();
-			result.adjuncts_for_contacts_summaries.resize(result.contacts_summaries.size(), ContactDescriptorSummaryAdjunct(adjunct_max_circle_radius_restrictions.size()));
+			result.adjuncts_for_contacts_summaries.resize(result.contacts_summaries.size(), ContactDescriptorSummaryAdjunct(adjunct_max_circle_radius_restrictions.size()*number_of_applications_of_preliminary_cuts));
 
 #ifdef VORONOTALT_OPENMP
 #pragma omp parallel
@@ -462,6 +630,8 @@ public:
 				RadicalTessellationContactConstruction::ContactDescriptor cd;
 				cd.contour.reserve(12);
 
+				RadicalTessellationContactConstruction::PreliminaryCuttingPlanes preliminary_cutting_planes;
+
 #ifdef VORONOTALT_OPENMP
 #pragma omp for
 #endif
@@ -470,27 +640,43 @@ public:
 					const ContactDescriptorSummary& cds=result.contacts_summaries[i];
 					if(cds.area>FLOATCONST(0.0))
 					{
-						ContactDescriptorSummaryAdjunct& cdsa=result.adjuncts_for_contacts_summaries[i];
-						Float prev_circle_radius_restriction=0.0;
-						for(UnsignedInt j=0;j<adjunct_max_circle_radius_restrictions.size();j++)
+						UnsignedInt lindex=0;
+						for(UnsignedInt k=0;k<number_of_applications_of_preliminary_cuts;k++)
 						{
-							const Float circle_radius_restriction=(max_circle_radius_restriction>FLOATCONST(0.0) ? std::min(adjunct_max_circle_radius_restrictions[j], max_circle_radius_restriction) : adjunct_max_circle_radius_restrictions[j]);
-							if(j==0 || (circle_radius_restriction>=prev_circle_radius_restriction)
-									|| (circle_radius_restriction<prev_circle_radius_restriction && cdsa.level_areas[j-1]>FLOATCONST(0.0)))
+							if(apply_preliminary_cuts_with_single_mask)
 							{
-								if(RadicalTessellationContactConstruction::construct_contact_descriptor(
-										spheres_container.populated_spheres(),
-										spheres_container.all_exclusion_statuses(),
-										cds.id_a,
-										cds.id_b,
-										spheres_container.all_colliding_ids()[cds.id_a],
-										circle_radius_restriction,
-										cd))
+								parameters_for_preliminary_cuts.prepare_input_for_preliminary_cuts(spheres_container.populated_spheres(), cds.id_a, cds.id_b, parameters_for_preliminary_cuts.single_mask, preliminary_cutting_planes);
+							}
+							else if(apply_preliminary_cuts_with_all_masks)
+							{
+								parameters_for_preliminary_cuts.prepare_input_for_preliminary_cuts(spheres_container.populated_spheres(), cds.id_a, cds.id_b, k, preliminary_cutting_planes);
+							}
+							ContactDescriptorSummaryAdjunct& cdsa=result.adjuncts_for_contacts_summaries[i];
+							Float prev_circle_radius_restriction=0.0;
+							for(UnsignedInt j=0;j<adjunct_max_circle_radius_restrictions.size();j++)
+							{
+								const Float circle_radius_restriction=(max_circle_radius_restriction>FLOATCONST(0.0) ? std::min(adjunct_max_circle_radius_restrictions[j], max_circle_radius_restriction) : adjunct_max_circle_radius_restrictions[j]);
+								if(j==0 || (circle_radius_restriction>=prev_circle_radius_restriction)
+										|| (circle_radius_restriction<prev_circle_radius_restriction && cdsa.level_areas[lindex-1].area>FLOATCONST(0.0)))
 								{
-									cdsa.level_areas[j]=cd.area;
+									if(RadicalTessellationContactConstruction::construct_contact_descriptor(
+											spheres_container.populated_spheres(),
+											spheres_container.all_exclusion_statuses(),
+											cds.id_a,
+											cds.id_b,
+											spheres_container.all_colliding_ids()[cds.id_a],
+											circle_radius_restriction,
+											preliminary_cutting_planes,
+											cd))
+									{
+										cdsa.level_areas[lindex].area=cd.area;
+									}
 								}
+								cdsa.level_areas[lindex].zone=k;
+								cdsa.level_areas[lindex].restriction=circle_radius_restriction;
+								lindex++;
+								prev_circle_radius_restriction=circle_radius_restriction;
 							}
-							prev_circle_radius_restriction=circle_radius_restriction;
 						}
 					}
 				}
@@ -499,6 +685,59 @@ public:
 
 		time_recorder.record_elapsed_miliseconds_and_reset("copy valid contact summaries");
 
+		if(with_tessellation_net)
+		{
+			UnsignedInt number_of_edges_from_subnets=0;
+			UnsignedInt number_of_vertices_from_subnets=0;
+			for(UnsignedInt i=0;i<ids_of_valid_pairs.size();i++)
+			{
+				const TessellationNet& subnet=possible_tessellation_subnets[ids_of_valid_pairs[i]];
+				number_of_edges_from_subnets+=subnet.tes_edges.size();
+				number_of_vertices_from_subnets+=subnet.tes_vertices.size();
+			}
+			TessellationNet redundant_net;
+			redundant_net.tes_edges.reserve(number_of_edges_from_subnets);
+			redundant_net.tes_vertices.reserve(number_of_vertices_from_subnets);
+			for(UnsignedInt i=0;i<ids_of_valid_pairs.size();i++)
+			{
+				const TessellationNet& subnet=possible_tessellation_subnets[ids_of_valid_pairs[i]];
+				redundant_net.tes_edges.insert(redundant_net.tes_edges.end(), subnet.tes_edges.begin(), subnet.tes_edges.end());
+				redundant_net.tes_vertices.insert(redundant_net.tes_vertices.end(), subnet.tes_vertices.begin(), subnet.tes_vertices.end());
+			}
+			std::sort(redundant_net.tes_edges.begin(), redundant_net.tes_edges.end());
+			std::sort(redundant_net.tes_vertices.begin(), redundant_net.tes_vertices.end());
+			result.tessellation_net.tes_edges.reserve(redundant_net.tes_edges.size()/2);
+			result.tessellation_net.tes_vertices.reserve(redundant_net.tes_vertices.size()/4);
+			for(UnsignedInt i=0;i<redundant_net.tes_edges.size();i++)
+			{
+				if(i==0 || !(redundant_net.tes_edges[i]==result.tessellation_net.tes_edges.back()))
+				{
+					result.tessellation_net.tes_edges.push_back(redundant_net.tes_edges[i]);
+				}
+			}
+			for(UnsignedInt i=0;i<redundant_net.tes_vertices.size();i++)
+			{
+				if(i==0 || !(redundant_net.tes_vertices[i]==result.tessellation_net.tes_vertices.back()))
+				{
+					result.tessellation_net.tes_vertices.push_back(redundant_net.tes_vertices[i]);
+					RadicalTessellationContactConstruction::TessellationVertex& tv=result.tessellation_net.tes_vertices.back();
+					tv.dist_min=FLOATCONST(0.0);
+					tv.dist_max=FLOATCONST(0.0);
+					if(tv.ids_of_spheres[3]<spheres_container.populated_spheres().size())
+					{
+						for(UnsignedInt j=0;j<4;j++)
+						{
+							const SimpleSphere& ss=spheres_container.populated_spheres()[tv.ids_of_spheres[j]];
+							const Float dist=distance_from_point_to_point(tv.position, ss.p)-ss.r;
+							tv.dist_min=((j==0 || dist<tv.dist_min) ? dist : tv.dist_min);
+							tv.dist_max=((j==0 || dist>tv.dist_max) ? dist : tv.dist_max);
+						}
+					}
+				}
+			}
+			time_recorder.record_elapsed_miliseconds_and_reset("assemble valid contacts tessellation net");
+		}
+
 		if(with_graphics)
 		{
 			result_graphics.contacts_graphics.resize(ids_of_valid_pairs.size());
@@ -582,6 +821,41 @@ public:
 			}
 
 			time_recorder.record_elapsed_miliseconds_and_reset("reassign ids in contacts at boundaries");
+
+			if(with_tessellation_net && count_of_redundant_contacts_in_periodic_box>0)
+			{
+				for(UnsignedInt i=0;i<result.tessellation_net.tes_vertices.size();i++)
+				{
+					RadicalTessellationContactConstruction::TessellationVertex& tv=result.tessellation_net.tes_vertices[i];
+					for(UnsignedInt j=0;j<4;j++)
+					{
+						if(tv.ids_of_spheres[j]!=null_id())
+						{
+							tv.ids_of_spheres[j]=tv.ids_of_spheres[j]%result.total_spheres;
+						}
+					}
+					tv.sort_ids_of_spheres();
+				}
+
+				std::sort(result.tessellation_net.tes_vertices.begin(), result.tessellation_net.tes_vertices.end());
+
+				for(UnsignedInt i=0;i<result.tessellation_net.tes_edges.size();i++)
+				{
+					RadicalTessellationContactConstruction::TessellationEdge& te=result.tessellation_net.tes_edges[i];
+					for(UnsignedInt j=0;j<3;j++)
+					{
+						if(te.ids_of_spheres[j]!=null_id())
+						{
+							te.ids_of_spheres[j]=te.ids_of_spheres[j]%result.total_spheres;
+						}
+					}
+					te.sort_ids_of_spheres();
+				}
+
+				std::sort(result.tessellation_net.tes_edges.begin(), result.tessellation_net.tes_edges.end());
+
+				time_recorder.record_elapsed_miliseconds_and_reset("reassign ids in contacts tessellation net at boundaries");
+			}
 		}
 
 		for(UnsignedInt i=0;i<result.contacts_summaries.size();i++)
