@@ -251,6 +251,16 @@ public:
 			output << obj.atom_name;
 		}
 	}
+
+	template<class SequentialContainer>
+	inline static void print_sequential_container_simply(const SequentialContainer& v, std::ostream& output) noexcept
+	{
+		for(typename SequentialContainer::const_iterator it=v.begin();it!=v.end();++it)
+		{
+			print((*it), output);
+		}
+	}
+
 private:
 	template<class ContactsContainer, class GroupedContactsContainer>
 	static void print_contacts_residue_level_or_chain_level_to_stream(
@@ -493,6 +503,43 @@ private:
 	{
 		output << obj.area << "\t" << obj.arc_length << "\t" << obj.distance << "\t" << obj.count;
 	}
+
+	inline static void print(const RadicalTessellationContactConstruction::TessellationEdge& te, std::ostream& output) noexcept
+	{
+		output << te.ids_of_spheres[0] << "\t" << te.ids_of_spheres[1] << "\t";
+		if(te.ids_of_spheres[2]==null_id())
+		{
+			output << "-1";
+		}
+		else
+		{
+			output << te.ids_of_spheres[2];
+		}
+		output << "\t" << te.length << "\n";
+	}
+
+	inline static void print(const RadicalTessellationContactConstruction::TessellationVertex& tv, std::ostream& output) noexcept
+	{
+		output << tv.ids_of_spheres[0] << "\t" << tv.ids_of_spheres[1] << "\t";
+		if(tv.ids_of_spheres[2]==null_id())
+		{
+			output << "-1";
+		}
+		else
+		{
+			output << tv.ids_of_spheres[2];
+		}
+		output << "\t";
+		if(tv.ids_of_spheres[3]==null_id())
+		{
+			output << "-1";
+		}
+		else
+		{
+			output << tv.ids_of_spheres[3];
+		}
+		output << "\t" << tv.position.x << "\t" << tv.position.y << "\t" << tv.position.z << "\t" << tv.dist_min << "\t" << tv.dist_max << "\n";
+	}
 };
 
 
